package com.holmos.webtest.junitextentions.parameters;

import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.junit.runners.model.FrameworkMethod;

import com.holmos.reflect.HolmosReflectCheck;
import com.holmos.webtest.exceptions.HolmosFailedError;
import com.holmos.webtest.junitextentions.annotations.ValueAssert;
import com.holmos.webtest.junitextentions.annotations.ValueVerify;
import com.holmos.webtest.utils.HolmosTypeCastUtils;
/**
 * 单个带有参数的方法，继承与junit的FrameWorkMethod类，参数的值可能来自于：<br>
 * 方法允许返回值
 * <li>excel</li>
 * <li>cvs</li>
 * <li>database</li>
 * <li>xml</li>
 * <li>properties</li>
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
public class ParameterFrameWorkMethod extends FrameworkMethod{
	/**记录是否运行成功*/
	private boolean succeed = false;
	/**当前的带参方法所在的执行序号,如果为-1则代表只执行一次*/
	private int index;
	/**参数列表*/
	private Object[] parameter;
	/**校验值列表*/
	private Object[] resultValue;
	/**为单个方法运行索特别设置的字段，以后如果找到好的解决方案的时候再行改掉*/
	private String methodName=null;
	
	private boolean useParameterAssert = false;
	
	public ParameterFrameWorkMethod(Method method,Object[]parameter,Object[] resultValue,int index){
		super(method);
		useParameterAssert = (method.getAnnotation(ValueAssert.class) == null && 
				method.getAnnotation(ValueVerify.class) == null )? false : true;
		this.parameter=parameter;
		this.resultValue = resultValue;
		this.index=index;
		this.methodName=method.getName();
	}
	@Override
	public String toString() {
		if(methodName!=null)return methodName;
		StringBuilder info=new StringBuilder(getName()+'<');
		if(null!=parameter)
		{
			for(Object value:parameter){
				if(null == value){
					info.append("null,");
				}
				if(value instanceof String){
					info.append("\""+value+"\""+",");
				}else if(value instanceof StringBuilder){
					info.append("\""+value.toString()+"\""+",");
				}else if(value instanceof Number){
					info.append(value+",");
				}else if(value instanceof Character){
					info.append("\'"+value.toString()+"\'"+",");
				}else if(value instanceof CharSequence){
					info.append("\""+value.toString()+"\""+",");
				}else{
					info.append(value.toString());
				}
			}
			info=new StringBuilder(info.substring(0, info.length()-1));
		}
		info.append(">["+(index+1)+']');
		return info.toString();
	}
	/**覆盖父类的执行方法,其实并没有做出一点修改,只是为了显示此处是做了修改,在此声明而已*/
	@Override
	public Object invokeExplosively(Object target, Object... parameter) throws Throwable {
		Object result=super.invokeExplosively(target, this.parameter);
		//如果super.invokeExplosively() 失败，就是抛出异常，那么succeed将还是false
		succeed = true;
		if(useParameterAssert){
			getActualReturnValue(getResultValueTypes(result));
			assertOrVerify(resultValue, result);
		}
		return result;
	}
	
	@SuppressWarnings("unchecked")
	private ArrayList<Class<?>> getResultValueTypes(Object result){
		ArrayList<Class<?>> types =new ArrayList<Class<?>>();
		if(null == result)//当返回值是 void 类型的时候
			return null;
		if(result instanceof List || result instanceof Array|| result instanceof Set){
			for(Object value :(Collection<Object>)result){
				types.add(value.getClass());
			}
		}else{
			throw new HolmosFailedError("holmos参数化校验的返回值不支持除 List,Array,Set之外的其他类型!如有需求，请联系作者!");
		}
		return types;
	}
	
	private Object[] getActualReturnValue(ArrayList<Class<?>>types){
		resultValue=(Object[]) HolmosTypeCastUtils.getParameterEndByBlank(resultValue);
		if(types.size()!=resultValue.length)
			throw new HolmosFailedError("测试方法   "+methodName+"()  中的待校验参数长度和参数源文件中的参数的长度不一致!");
		Object[]myParameter=new Object[types.size()];
		for(int i=0;i<resultValue.length;i++){
			myParameter[i]=HolmosTypeCastUtils.castObjectAsType(types.get(i), resultValue[i]);
		}return myParameter;
	}
	
	private void assertOrVerify(Object expected,Object actual){
		if(getMethod().getReturnType() != Void.TYPE){
			boolean verify = getMethod().getAnnotation(ValueAssert.class)!=null?false:true;
			if(verify){
				verifyCheck(expected, actual);
			}else{
				assertCheck(expected, actual);
			}
		}
	}
	
	private void verifyCheck(Object expected,Object actual){
		try{
			HolmosReflectCheck.assertEquals(methodName+"校验失败!", expected, actual);
		}catch(Exception e){
			
		}
	}
	
	private void assertCheck(Object expected,Object actual){
		HolmosReflectCheck.assertEquals(methodName+"校验失败!", expected, actual);
	}
	
	public String getMethodName() {
		return methodName;
	}
	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}
	public boolean isSucceed() {
		return succeed;
	}
	public void setSucceed(boolean succeed) {
		this.succeed = succeed;
	}
}
